# -*- coding: utf-8 -*-
require 'sidfm_controller_support'

class SidfmHistoriesController < ApplicationController
  unloadable
  layout 'sidfm'
  self.main_menu = false
  
  helper :sidfm
  protect_from_forgery with: :exception

  include SidfmControllerSupport

  before_action :update_session_updated_at
  before_action :create_common_data
  before_action :get_sidfm_roles
  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session, only: [ :index ]
  before_action :check_roles 
  before_action :prevent_access, only: [ :edit_comment, :get_data_for_bulk_update, :bulk_update_comment ]
  before_action -> { get_summary_table_lines('histories') }
 
  private
  
  def check_roles
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    @role_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = if @role_admin then
                           nil
                         else
                           SidfmdCategoriesUser.where(:user_id => User.current.id)
                                               .select(:sidfmd_category_id)
                                               .order(:sidfmd_category_id)
                                               .distinct
                                               .pluck(:sidfmd_category_id)
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host)
                                           .where(:sidfmd_category_id => @user_category_ids)
                                           .where(:sidfmd_hosts => { :flg_delete => false })
                                           .select(:sidfmd_host_id)
                                           .order(:sidfmd_host_id)
                                           .distinct
                                           .pluck(:sidfmd_host_id)
                     end
  end

  def prevent_access
    if !@sidfm_roles.include?(:use) or @sidfm_roles.include?(:host_limited_admin) then #管理者と報告者のみアクセス可能
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
  end

  def check_param_journal_ids(journal_ids)
    #管理者権限以外の時に、journal_idsに自分のカテゴリ範囲外のものが含まれていないかチェック
    !SidfmdIssue.joins(:issue)
                .joins(%| INNER JOIN journals ON journals.journalized_id = issues.id |) 
                .where.not(:sidfmd_issues => { :sidfmd_host_id => @user_host_ids })
                .where(:journals => { :id => journal_ids })
                .exists?
  end

  # パラメータ解析
  def parse_params
    @histories = {}
    pages_params = params.fetch(:pages, {}).permit(:order, :offset, :per_page, :filters, :checked, :all_checked)

    @histories[:order] = (pages_params[:order] || 'created_on_desc')
    @histories[:filters] = JSON.parse((pages_params[:filters] || '{}'), symbolize_names: true)
    @histories[:offset] = (pages_params[:offset] || 0).to_i
    @histories[:per_page] = (@summary_table_lines || pages_params[:per_page] || 10).to_i
    @histories[:checked] = JSON.parse(pages_params[:checked] || '[]')
    @histories[:all_checked] = pages_params[:all_checked] || ''
  end

  def create_content
    @histories[:all_content] = SidfmdIssue.joins(:issue)
                                          .joins(%| INNER JOIN journals ON journals.journalized_id = issues.id |)
                                          .joins(%| LEFT OUTER JOIN journal_details ON journals.id = journal_details.journal_id |)
                                          .joins(%| INNER JOIN sidfmd_hosts ON sidfmd_hosts.id = sidfmd_issues.sidfmd_host_id |)
                                          .joins(%| INNER JOIN users ON journals.user_id = users.id |)
                                          .joins(%| LEFT OUTER JOIN email_addresses ON users.id = email_addresses.user_id |) 
                                          .where(:sidfmd_hosts => { :flg_delete => false })
    if !@role_admin then
      @histories[:all_content] = @histories[:all_content].where(:sidfmd_issues => { :sidfmd_host_id => @user_host_ids })
                                                         #.where(:users => { 
                                                         #  :id => SidfmdCategoriesUser.where(:sidfmd_category_id => @user_category_ids)
                                                         #                             .select(:user_id) 
                                                         #})
    end
  end

  FILTER_KEYS = {}

  #フィルターの初期化
  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      FILTER_KEYS[:roles] = { :ids => [], :is_except => false }
      FILTER_KEYS[:users] =  { :ids => [], :is_except => false, :is_except_anonymous => false }
      FILTER_KEYS[:hosts] = { :ids => [], :is_except => false }
      FILTER_KEYS[:categories] = { :ids => [], :is_except => false, :is_and => false }
      FILTER_KEYS[:date] = '::'
      FILTER_KEYS[:opes] = { :targets => [], :is_except => false }
      FILTER_KEYS[:status] = '0'
      FILTER_KEYS[:comment] = { :text => '', :is_except => false }
      FILTER_KEYS[:vuls] = ''
    end  
  end

  def check_filters
    init_filter_keys
    FILTER_KEYS.each do |key, val|
      unless @histories[:filters].has_key?(key) then
        @histories[:filters][key] = val
      end
    end
    @histories[:filters][:roles][:ids] = @histories[:filters][:roles][:ids].map(&:to_i)
    @histories[:filters][:users][:ids] = @histories[:filters][:users][:ids].map(&:to_i)
    @histories[:filters][:hosts][:ids] = @histories[:filters][:hosts][:ids].map(&:to_i)
    @histories[:filters][:categories][:ids] = @histories[:filters][:categories][:ids].map(&:to_i)
    
    @default_filters = FILTER_KEYS
  end

  def sort_content
    separate_char_index = @histories[:order].rindex('_')
    col_name = @histories[:order].slice(0..separate_char_index - 1)
    order = @histories[:order].slice(separate_char_index + 1..@histories[:order].size - 1)
    case col_name
    when 'created_on' then
      @histories[:all_content] = @histories[:all_content].order(%| journals.created_on #{order == 'asc' ? 'ASC' : 'DESC'} |)
    when 'login_name' then
      @histories[:all_content] = @histories[:all_content].order(%| LOWER(users.login COLLATE "C") #{order == 'asc' ? 'ASC' : 'DESC'} |)
    when 'operation' then
      @histories[:all_content] = @histories[:all_content].order(%|
        CASE journal_details.prop_key
          WHEN 'status_id' THEN 1
          WHEN 'due_date'THEN 2
          ELSE 3
        END
        #{order == 'asc' ? 'ASC' : 'DESC'} 
      |).order(%| journals.created_on #{order == 'asc' ? 'DESC' : 'ASC'} |)
    when 'hostname' then
      @histories[:all_content] = @histories[:all_content].order(%|
        LOWER(
          (CASE WHEN sidfmd_hosts.ext_name IS NULL THEN sidfmd_hosts.hostname ELSE sidfmd_hosts.ext_name END) COLLATE "C"
        ) #{order == 'asc' ? 'ASC' : 'DESC'} 
      |)
    when 'vul_title' then
      @histories[:all_content] = @histories[:all_content].order(%| abs(sidfmd_issues.html_id) #{order == 'asc' ? 'ASC' : 'DESC'}|)
                                                         .order(%| LOWER(sidfmd_issues.title COLLATE "C") #{order == 'asc' ? 'ASC' : 'DESC'} |)
    when 'status' then
      @histories[:all_content] = @histories[:all_content].order(%| issues.status_id #{order == 'asc' ? 'ASC' : 'DESC'} |)
                                                         .order(%| journals.created_on #{order == 'asc' ? 'DESC' : 'ASC'} |)
    when 'due_date' then
      @histories[:all_content] = @histories[:all_content].order(%| issues.due_date #{order == 'asc' ? 'ASC' : 'DESC'} |)
    end 
    if col_name != 'created_on' then
      @histories[:all_content] = @histories[:all_content].order(%| journals.created_on #{order == 'asc' ? 'DESC' : 'ASC'} |)
    end
    if col_name != 'login_name' then
      @histories[:all_content] = @histories[:all_content].order(%| LOWER(users.login COLLATE "C") #{order == 'asc' ? 'ASC' : 'DESC'} |)
    end
    @histories[:all_content] = @histories[:all_content].order(%| journals.id #{order == 'asc' ? 'ASC' : 'DESC'} |)
  end

  
  DAY_SECONDS = 24 * 60 * 60
  
  def apply_filters
    #権限検索
    if @histories[:filters][:roles][:ids].any? then
      @histories[:all_content] = if @histories[:filters][:roles][:is_except] then
                                   @histories[:all_content].where(
                                     :users => {:id =>  Member.joins(:member_roles)
                                                              .where.not(:member_roles => {:role_id => @histories[:filters][:roles][:ids]})
                                                              .select(:user_id)}
                                   )
                                 else
                                   @histories[:all_content].where(
                                     :users => {:id =>  Member.joins(:member_roles)
                                                              .where(:member_roles => {:role_id => @histories[:filters][:roles][:ids]})
                                                              .select(:user_id)}
                                   )
                                 end
    end

    #ユーザ検索
    if @histories[:filters][:users][:is_except_anonymous] then
      @histories[:all_content] = @histories[:all_content].where.not(:users => {:type => 'AnonymousUser'})
    end
    if @histories[:filters][:users][:ids].any? then
      @histories[:all_content] = if @histories[:filters][:users][:is_except] then
                                   @histories[:all_content].where.not(:users => {:id => @histories[:filters][:users][:ids]})
                                 else
                                   @histories[:all_content].where(:users => {:id => @histories[:filters][:users][:ids]})
                                 end
    end

    #ホスト検索
    if @histories[:filters][:hosts][:ids].any? then
      @histories[:all_content] = if @histories[:filters][:hosts][:is_except] then
                                   @histories[:all_content].where.not(:sidfmd_hosts => {:id => @histories[:filters][:hosts][:ids]})
                                 else
                                   @histories[:all_content].where(:sidfmd_hosts => {:id => @histories[:filters][:hosts][:ids]})
                                 end
    end

    #カテゴリ検索
    if @histories[:filters][:categories][:ids].any? then
      target_host_ids = []
      if @histories[:filters][:categories][:is_and] then
        SidfmdHost.where(
          :id => SidfmdCategoriesHost.where(:sidfmd_category_id => @histories[:filters][:categories][:ids]).select(:sidfmd_host_id)
        ).where(:flg_delete => false)
        .each do |host|
          host_cat_ids = host.sidfmd_categories.pluck(:id)
          if (@histories[:filters][:categories][:ids] - host_cat_ids).empty? then
            target_host_ids << host.id
          end
        end
      else
        target_host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => @histories[:filters][:categories][:ids])
                                              .select(:sidfmd_host_id)
      end

      @histories[:all_content] = if @histories[:filters][:categories][:is_except] then
                                   @histories[:all_content].where.not(:sidfmd_hosts => { :id => target_host_ids })
                                 else
                                   @histories[:all_content].where(:sidfmd_hosts => {:id => target_host_ids})
                                 end
    end

    #期間指定検索
    if @histories[:filters][:date] != '::' then
      if /^(in|rev):(\d{4}\/\d{2}\/\d{2}|):(\d{4}\/\d{2}\/\d{2}|)$/ =~ @histories[:filters][:date] then
        orig_flg_rev = Regexp.last_match(1)
        orig_begin_date = Regexp.last_match(2)
        orig_end_date = Regexp.last_match(3)
        if orig_flg_rev.size > 0 and orig_begin_date.size == 0 and orig_end_date.size == 0 then
          @histories[:filters][:date] = '::'
        else
          flg_rev = orig_flg_rev == 'rev' ? true : false
          begin_date = orig_begin_date.size > 0 ? Date.strptime(orig_begin_date, '%Y/%m/%d') : nil
          end_date = orig_end_date.size > 0 ? Date.strptime(orig_end_date, '%Y/%m/%d') : nil
          if flg_rev then
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              @histories[:all_content] = @histories[:all_content].where(
                'journals.created_on <= ? OR journals.created_on >= ?', begin_date, end_date
              )
            elsif begin_date then
              begin_date = begin_date.to_time
              @histories[:all_content] = @histories[:all_content].where(
                'journals.created_on <= ?', begin_date
              )
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              @histories[:all_content] = @histories[:all_content].where(
                'journals.created_on >= ?', end_date
              )
            else
              @histories[:filters][:date] = '::' # last resort
            end
          else
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              @histories[:all_content] = @histories[:all_content].where(
                'journals.created_on BETWEEN ? AND ?', begin_date, end_date
              )
            elsif begin_date then
              begin_date = begin_date.to_time
              @histories[:all_content] = @histories[:all_content].where(
                'journals.created_on >= ?', begin_date
              )
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              @histories[:all_content] = @histories[:all_content].where(
                'journals.created_on <= ?', end_date
              )
            else
              @histories[:filters][:date] = '::' # last resort
            end
          end
        end
      else
        @histories[:filters][:date] = '::'
      end
    end
 
    #操作検索
    if @histories[:filters][:opes][:targets].any? then
      targets = @histories[:filters][:opes][:targets].select{|target| target == 'status_id' or target == 'due_date' } #フィルタ
      @histories[:all_content] = if @histories[:filters][:opes][:is_except] then
                                   @histories[:all_content].where(%|
                                     journal_details.prop_key IS NULL OR journal_details.prop_key NOT IN (?)
                                   |, targets)
                                 else
                                   @histories[:all_content].where(:journal_details => { :prop_key => targets })
                                 end
    end

    # 対応状況
    if /\A(\d):(\d)\z/ =~  @histories[:filters][:status] then
      status1 = Regexp.last_match(1).to_i
      status2 = Regexp.last_match(2).to_i
      logger.info( [ status1, status2 ])
      @histories[:all_content] = @histories[:all_content].where(
        :issues => { :status_id => [ status1, status2 ] }
      )
    elsif @histories[:filters][:status]!= '0' then
      @histories[:all_content] = @histories[:all_content].where(
        :issues => { :status_id => @histories[:filters][:status].to_i }
      )
    end

    #コメント検索
    if @histories[:filters][:comment][:text].size > 0 or @histories[:filters][:comment][:is_except] then
      @histories[:all_content] = if @histories[:filters][:comment][:is_except] then
        @histories[:all_content].where.not(%| journals.notes ILIKE ? |, "%#{ActiveRecord::Base.send(:sanitize_sql_like, @histories[:filters][:comment][:text])}%")
      else
        @histories[:all_content].where(%| journals.notes ILIKE ? |, "%#{ActiveRecord::Base.send(:sanitize_sql_like, @histories[:filters][:comment][:text])}%")
      end 
    end
 
    #脆弱性検索
    if @histories[:filters][:vuls].size > 0 then
      if /\A\s*(\d+)\s*\z/ =~ @histories[:filters][:vuls] then
        html_id = Regexp.last_match(1)
        @histories[:all_content] = @histories[:all_content].where(%|
          sidfmd_issues.html_id = ? OR sidfmd_issues.title ILIKE ? 
        |, html_id.to_i, "%#{ActiveRecord::Base.send(:sanitize_sql_like, html_id.to_s.strip)}%")
      elsif /\A\s*KB(\d+)\s*\z/i =~ @histories[:filters][:vuls] then
        kbid = -Regexp.last_match(1).to_i
        @histories[:all_content] = @histories[:all_content].where(%|
          CAST(sidfmd_issues.html_id AS text) ILIKE ? OR sidfmd_issues.title ILIKE ?
        |, kbid.to_s, "%#{ActiveRecord::Base.send(:sanitize_sql_like, @histories[:filters][:vuls].strip)}%")
      else
        @histories[:all_content] = @histories[:all_content].where(%|
          sidfmd_issues.title ILIKE ? OR sidfmd_issues.id IN (SELECT DISTINCT sidfmd_issue_id FROM sidfmd_issue_cves WHERE cve_id ILIKE ?)
        |,"%#{ActiveRecord::Base.send(:sanitize_sql_like, @histories[:filters][:vuls].strip)}%", "%#{ActiveRecord::Base.send(:sanitize_sql_like, @histories[:filters][:vuls].strip)}%")
      end
    end
=begin    #(title)
    if @histories[:filters][:vuls][:title].size > 0 then
      @histories[:all_content] = @histories[:all_content].where(%| sidfmd_issues.title ILIKE ? |, "%#{@histories[:filters][:vuls][:title].strip}%")
    end
    #脆弱性検索
    #(SIDfmID(html_id))
    if @histories[:filters][:vuls][:html_id].size > 0 then
      html_id_str = if /\A\s*KB(\d+)\s*\z/i =~ @histories[:filters][:vuls][:html_id] then
                      Regexp.last_match(1)
                    else
                      @histories[:filters][:vuls][:html_id]
                    end
      @histories[:all_content] = @histories[:all_content].where(%| CAST(sidfmd_issues.html_id AS text) ILIKE ? |, "%#{html_id_str.strip}%")
    end
    #脆弱性検索
    #(cve_id)
    if @histories[:filters][:vuls][:cve_id].size > 0 then
      @histories[:all_content] = @histories[:all_content].where(
        :sidfmd_issues => { :id => SidfmdIssueCve.where('cve_id ILIKE ?', "%#{@histories[:filters][:vuls][:cve_id].strip}%")
                                                 .select(:sidfmd_issue_id) 
                          }
      )
    end
=end

    sort_content
    @histories[:all_content]
  end

  def filter_content(is_limit_data = true)
    check_filters
    @histories[:content_filtered] = apply_filters
    @histories[:total_size] = @histories[:content_filtered].count
    @histories[:content] =
       @histories[:content_filtered].select(
                                      'journals.id AS journal_id',
                                      'journals.created_on AS journal_created_on',
                                      'users.id AS user_id',
                                      'users.login AS login_name',
                                      'email_addresses.address AS email_address',
                                      " CASE 
                                         WHEN users.type = 'AnonymousUser' THEN
                                           ''
                                         ELSE
                                           (users.lastname || ' ' || users.firstname)
                                         END
                                         AS user_fullname
                                      ",
                                      %| CASE 
                                         WHEN users.type = 'AnonymousUser' THEN
                                           true
                                         ELSE
                                           false
                                         END
                                         AS is_anonymous
                                      |,
                                      %| CASE journal_details.prop_key 
                                           WHEN 'status_id' THEN 'ステータス変更' 
                                           WHEN 'due_date' THEN '対策予定日の変更'
                                           ELSE NULL
                                         END
                                         AS operation
                                      |,
                                      'journal_details.old_value AS old_value', #status_idの場合もtext型なので注意
                                      'journal_details.value AS current_value',
                                      'journals.notes AS comment',
                                      'sidfmd_hosts.hostname AS hostname',
                                      'sidfmd_hosts.ext_name AS ext_name',
                                      %|
                                        CASE 
                                        WHEN sidfmd_hosts.ext_name IS NULL THEN sidfmd_hosts.hostname
                                        ELSE sidfmd_hosts.ext_name
                                        END
                                        AS display_name
                                      |,
                                      'sidfmd_hosts.id AS host_id',
                                      'sidfmd_issues.html_id AS html_id',
                                      'sidfmd_issues.title AS vul_title',
                                      'sidfmd_issues.short_description AS vul_short_description',
                                      'issues.status_id AS status_id',
                                      'issues.due_date AS due_date' 
                                    )
    if is_limit_data then
      @histories[:content] = @histories[:content].offset(@histories[:offset])
                                                 .limit(@histories[:per_page])
    end
  end

  def get_roles
    role_names = [ 'SIDfmVM管理者','SIDfmVM報告者', 'SIDfmVM利用者', 'SIDfmVM閲覧者' ]
    Role.where(:name => role_names)
        .map{|r| {:id => r.id, :name => r.name } }
        .sort_by{|role| role_names.index(role[:name])}
  end
 
  def get_user_categories
    sort_categories = ->(categories){
      categories.order(%|
        CASE WHEN is_system = true THEN 10 ELSE 100 END
      |)
      .order(%|
        CASE WHEN is_system = true THEN CAST(id AS text) ELSE LOWER(name COLLATE "C") END
      |)
    }
    user_categories = if @role_admin then
                        sort_categories.(SidfmdCategory.all)
                      else
                        sort_categories.(SidfmdCategory.where("is_system = 'true' OR id IN (?)", @user_category_ids))
                      end
  end

  def create_summary_history_html(history, **opts)
    data = {
      :chkbox => view_context.create_summary_history_chkbox(history.journal_id, opts[:checked]),
      :created_on => view_context.create_summary_history_created_on(history.journal_created_on),
      :login_name => view_context.create_summary_history_user(history.login_name, history.user_fullname, history.email_address, history.is_anonymous),
      :operation => view_context.create_summary_history_operation(history.operation, history.old_value, history.current_value),
      :comment => view_context.create_summary_history_comment(history.comment, history.comment == HISTORY_COMMENT_DELETED_NOTE, history.journal_id, @sidfm_roles),
      :vul_title => view_context.create_summary_history_vul_title(history.vul_title, history.vul_short_description, history.host_id, history.html_id),
      :hostname => view_context.create_summary_history_hostname(
        history.hostname,
        history.ext_name,
        history.display_name,
        history.host_id,
        history.html_id
      ),
      :status => view_context.create_summary_history_status(history.status_id),
      :due_date => view_context.create_summary_history_due_date(history.due_date)
    }
  end

  public

  def index
    parse_params
    create_content
    filter_content
    @roles = get_roles
    @user_categories = get_user_categories
    @deleted_comment = HISTORY_COMMENT_DELETED_NOTE
  end

  def bulk_update_comment
    f_params = params.permit(:journal_ids, :comment, :comment_delete_flg)#, :page_info)
    begin
      journal_ids = JSON.parse(f_params[:journal_ids] || '[]')
      #page_info = JSON.parse(f_params[:page_info])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    if journal_ids.empty? then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    unless @role_admin then
      unless check_param_journal_ids(journal_ids) then
        render :layout => false, :json => { :err_message => '不正な操作です。※ユーザのカテゴリが変更されていないかご確認下さい。' } and return
      end
    end
    begin
      target_journals = Journal.where(:id => journal_ids)
      if f_params[:comment_delete_flg] then
        target_journals.update_all(:notes => HISTORY_COMMENT_DELETED_NOTE)
      elsif f_params[:comment] and f_params[:comment].size > 0 then
        target_journals.update_all(:notes => f_params[:comment])
      else
        render :layout => false, :json => { :err_message =>  '変更に失敗しました。' } and return
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message =>  '変更に失敗しました。' } and return
    end
    #dump = {
    #  :action => :index,
    #  :'pages[offset]' => page_info["offset"],
    #  :'pages[per_page]' => page_info["per_page"],
    #  :'pages[filters]' => JSON.generate(page_info["filters"]),
    #  :'pages[order]' => page_info["order"]
    #}
    #redirect_to dump
    render :layout => false, :json => { :err_message => '' }
  end

  CSV_BOM = "\xff\xfe".force_encoding('UTF-16LE').freeze
  MAX_CELL_LENGTH = 30000
  MAX_LINE_NUM = 254 # =最大改行数は253

  def read_download_data
    GC.start
    f_params = params.permit(:target, :ids, :executed_size, :process, :order, :filters)
    if !f_params[:ids] or !f_params[:target] or !f_params[:order] or !f_params[:filters] or !f_params[:executed_size] or !f_params[:process] then
      logger.info('params error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    executed_size = f_params[:executed_size].to_i

    begin 
      journal_ids = JSON.parse(f_params[:ids])
      filters = JSON.parse(f_params[:filters], symbolize_names: true)
    rescue JSON::ParserError => e
      logger.info('json parser error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    if ![ 'json', 'tsv' ].include?(f_params[:target]) then
      logger.info('file target error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
  
    
    @histories = {}
    @histories[:order] = f_params[:order]
    @histories[:filters] = filters
    create_content
    filter_content(false)
  
    status_names = {
      SidfmdIssue::STATUS_NOTCONFIRMED => '未対策',
      SidfmdIssue::STATUS_ONGOING => '対策中',
      SidfmdIssue::STATUS_FIXED => '対策済',
      SidfmdIssue::STATUS_NOTAFFECT => '影響無'
    }
   
    case f_params[:target]
    when 'json' then
      
      if f_params[:process] == 'init' then
        @histories[:content] = @histories[:content].where(:journals => {:id => journal_ids }) if journal_ids.any?
        journal_ids = @histories[:content].map{|j| j.journal_id }
        separate_char_index = @histories[:order].rindex('_')
        col_name = @histories[:order].slice(0..separate_char_index - 1)
        order_type = @histories[:order].slice(separate_char_index + 1..@histories[:order].size - 1) 
        init_json_summary_table_progress_download(
          :ids =>journal_ids, 
          :order_col_name => col_name, 
          :order_type => order_type, 
          :filters => filters, 
          :max_download_size => Sidfm.get_history_max_download_size
        )
      else
        create_json_data = Proc.new{|id|
          history = @histories[:content].find_by(:journals => {:id => id })
          next unless history
          item = {}
          item[:journal_id] = history.journal_id
          item[:created_on] = history.journal_created_on.strftime('%Y/%m/%d %H:%M:%S')
          item[:user] = {}
          item[:user][:id] = history.user_id
          item[:user][:login] = history.is_anonymous ? '削除済みユーザ' : history.login_name
          item[:user][:fullname] = history.is_anonymous ? '' : history.user_fullname
          item[:user][:email_address] = history.is_anonymous ? '' : history.email_address
          item[:operation] = {
            :title => history.operation ? history.operation : '',
            :content => if history.operation == 'ステータス変更' and history.old_value and history.current_value then
                          status_names[history.old_value.to_i] + ' から ' + status_names[history.current_value.to_i] + 'へ'
                        elsif history.operation == '対策予定日の変更' then
                          (history.old_value ? history.old_value : '未設定') + ' から ' + (history.current_value ? history.current_value : '未設定') + 'へ'
                        else
                          ''
                        end                         
          }
          item[:comment] = history.comment
          item[:vul] = {
            :title => "[#{history.html_id > 0 ? history.html_id : "KB#{-history.html_id}"}] #{history.vul_title}",
            :short_description => history.vul_short_description,
            :url => url_for(
                      :controller => :sidfm_vulnerability,
                      :action => :index,
                      :id => history.html_id,
                      :'form[history]' => [ history.host_id, history.html_id ].to_json
                    )
          }
          item[:host] = {
            :hostname => history.hostname,
            :ext_name => history.ext_name,
            :display_name => history.display_name,
            :url => url_for(
                      :controller => :sidfm_host,
                      :action => :index,
                      :id => history.host_id,
                      :'form[history]' => [ history.html_id, history.host_id ].to_json
                    )
          }
          item[:status] = status_names[history.status_id]
          item[:due_date] = history.due_date ? history.due_date.strftime('%Y/%m/%d') : ''
          item
        }
        json_tsv_summary_table_progress_download(journal_ids, create_json_data, executed_size)
      end 
    when 'tsv' then
      header = [ '日時', 'ログイン名', 'メールアドレス', '操作', '操作内容', 'コメント', '脆弱性', 'ホスト名', '対応状況', '対策予定日' ]
      if f_params[:process] == 'init' then
        @histories[:content] = @histories[:content].where(:journals => {:id => journal_ids }) if journal_ids.any?
        journal_ids = @histories[:content].map{|j| j.journal_id }
        init_tsv_summary_table_progress_download(journal_ids, header, Sidfm.get_history_max_download_size)
      else
        create_tsv_data = Proc.new{|id|
          history = @histories[:content].find_by(:journals => {:id => id })
          item = []
          item << history.journal_created_on.strftime('%Y/%m/%d %H:%M:%S')
          item << (history.is_anonymous ? '削除済みユーザ' : history.login_name)
          item << (history.is_anonymous ? '' : history.email_address)
          item << (history.operation ? history.operation : '')
          ope_content = if history.operation == 'ステータス変更' and history.old_value and history.current_value then
                          status_names[history.old_value.to_i] + ' から ' + status_names[history.current_value.to_i] + 'へ'
                        elsif history.operation == '対策予定日の変更' then
                          (history.old_value ? history.old_value : '未設定') + ' から ' + (history.current_value ? history.current_value : '未設定') + 'へ'
                        else
                          ''
                        end
          item << ope_content
          comment = history.comment
          split_comment_by_line = comment.split("\n")
          if split_comment_by_line.size > MAX_LINE_NUM then
            comment = split_comment_by_line.slice(0..MAX_LINE_NUM - 1).join("\n")
          end
          if comment.size > MAX_CELL_LENGTH then
            comment = comment.slice(0..MAX_CELL_LENGTH - 1)
          end
          item << comment
          item << "[#{history.html_id > 0 ? history.html_id : "KB#{-history.html_id}"}] #{history.vul_title}"
          item << history.display_name
          item << status_names[history.status_id]
          item << (history.due_date ? history.due_date.strftime('%Y/%m/%d') : '')
          item
        }
        json_tsv_summary_table_progress_download(journal_ids, create_tsv_data, executed_size)
      end
    end
  end

  def download
    f_params = params.fetch(:download, {}).permit(:format)
    case f_params[:format]
    when 'json' then
      send_summary_table_json_file("vmlist-histories-json-#{Time.current.strftime('%Y%m%d%H%M%S')}.json")
    when 'tsv' then
      send_summary_table_tsv_file("vmlist-histories-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv")
    end
  end

  #api
  def edit_comment
    f_params = params.permit(:journal_id, :comment, :comment_delete_flg)
    if !f_params[:journal_id] then
      render :layout => false, :json => {:err_message => '不正な操作です。'} and return 
    end

    comment = f_params[:comment]
    begin
      unless @role_admin then
        unless check_param_journal_ids([ f_params[:journal_id].to_i ]) then
          render :layout => false, :json => { :err_message => '不正な操作です。※ユーザのカテゴリが変更されていないかご確認下さい。' } and return
        end
      end 
      journal = Journal.find_by(:id => f_params[:journal_id].to_i)
      if journal then
        if f_params[:comment_delete_flg] == 'true' then
          journal.update(:notes => HISTORY_COMMENT_DELETED_NOTE)
          comment = ''
        elsif comment and comment.size > 0 then
          journal.update(:notes => comment)
        else
          render :layout => false, :json => {:err_message => '変更に失敗しました。'} and return
        end
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => {:err_message => '変更に失敗しました。'} and return
    end

    render :layout => false, :json => {:err_message => '', :comment => comment}

  end

  def get_data_for_bulk_update
    f_params= params.permit(:journal_ids)
    @histories = {}
    journal_ids = JSON.parse(f_params[:journal_ids] || '[]')
    create_content
    r = @histories[:all_content].where(:journals => { :id => journal_ids })
                                .limit(1000)
                                .select(
                                  'journals.notes AS comment',
                                  %| CASE WHEN users.type = 'AnonymousUser' THEN '削除済みユーザ' ELSE users.login END AS login |,
                                  'email_addresses.address AS address',
                                  " CASE 
                                      WHEN users.type = 'AnonymousUser' THEN
                                        ''
                                      ELSE
                                        (users.lastname || ' ' || users.firstname)
                                      END
                                      AS user_fullname
                                   ",
                                    %| CASE 
                                       WHEN users.type = 'AnonymousUser' THEN
                                         true
                                       ELSE
                                         false
                                       END
                                       AS is_anonymous
                                    |
                                )
    render :layput => false, :json => r
  end

  #def api_get_display_summary_rows
  #  GC.start
  #  f_params = params.permit(:ids, :checked)
  #  journal_ids = JSON.parse(f_params[:ids])
  #  checked = JSON.parse(f_params[:checked]).map(&:to_i)
  #  @histories = {}
  #  create_content
  #  ret = {}
  #  @histories[:all_content].where(:journals => { :id => journal_ids }).each do |history|
  #    ret[history.journal_id] = create_summary_history_html(history, { :checked =>  checked })
  #  end
  #
  #  render :layout => false, :json => ret
  #end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page)
    parse_params
    @histories[:per_page] = f_params[:per_page].to_i
    create_content
    filter_content
    ret = {
      :rows => @histories[:content].map{|history|
        create_summary_history_html(history, { :checked =>  @histories[:checked] })
      },
      :total =>  @histories[:content_filtered].count,
      :all_ids => @histories[:content_filtered].select('journals.id AS journal_id').map{|j| j.journal_id},
      :sidfm_roles => @sidfm_roles,
      :user_cats => @user_category_ids
    }
    render :layout => false, :json => ret
  end

end
