# -*- coding: utf-8 -*-
require_dependency 'user'

module SidfmPatches
  module UserPatch
    extend ActiveSupport::Concern

    included do
      unloadable
      has_and_belongs_to_many :sidfmd_categories
      has_many :sidfmd_rule_a_mail_categories, dependent: :destroy, foreign_key: 'from_user_id'
      has_many :sidfmd_rule_a_mail_users, dependent: :destroy
      has_many :sidfmd_rule_chp_user_subs, dependent: :destroy
      has_one :oic_session, dependent: :destroy     

      #パスワードポリシー
      validate do
        errors.add(:password) if !check_password_with_password_policy
      end

      #controller側でも使用する
      def validate_password_min_length(password)
        r = true
        min_length = Setting.password_min_length.to_i
        r = false if password.length < min_length
        return r
      end

      def validate_password_symbols(password) 
        r = true
        available_symbols = Sidfm.get_password_available_symbols
        password.gsub(/[a-zA-Z]|[0-9]/, '').each_char do |c|
          if !available_symbols.include?(c) then
            r = false
            break
          end
        end
        return r
      end

      def is_chars_include?(password, regexp)
        password_len = password.length
        #check_password = password.gsub(/#{regexp}/, '')
        check_password = password.gsub(Regexp.new(regexp), '')
        if password_len == check_password.length then #文字数が同じ == 文字が含まれていない
          return false
        else
          return true
        end
      end

      def validate_password_char_types(password)
        r = true
        available_char_types = Sidfm.get_password_available_char_types
        available_symbols = Sidfm.get_password_available_symbols
        is_chars_mix_needed = Sidfm.get_is_password_char_type_mix_needed
        available_char_types = available_char_types.split('/')
        is_en = available_char_types.include?('en')
        is_num = available_char_types.include?('num')
        is_sym = available_char_types.include?('sym')
        regxp_symbols = available_symbols.split('').map{|sym| 
          if "\$()-^|[]+*{}.?/".include?(sym) then
            '\\' + sym
          else
            sym
          end
        }.join
        regxp_symbols = '[' + regxp_symbols + ']'
        #使用許可されていない文字が入っているかチェック
        if !is_en then
          is_chars_included = is_chars_include?(password, '[a-zA-Z]')
          r = false if is_chars_included
        end
        if !is_num then
          is_chars_included = is_chars_include?(password, '[0-9]')
          r = false if is_chars_included
        end
        if !is_sym && available_symbols.size > 0 then
          is_chars_included = is_chars_include?(password, regxp_symbols)
          r = false if is_chars_included
        end
        #文字種の混在チェック
        if is_chars_mix_needed then
          if is_en then
            is_chars_included = is_chars_include?(password, '[a-zA-Z]')
            r = false if !is_chars_included
          end

          if is_num then
            is_chars_included = is_chars_include?(password, '[0-9]')
            r = false if !is_chars_included
          end
           
          if is_sym && available_symbols.size > 0 then
            is_chars_included = is_chars_include?(password, regxp_symbols)
            r = false if !is_chars_included
          end
        else
          check_password = password
          check_password = check_password.gsub(/[a-zA-Z]/, '') if is_en
          check_password = check_password.gsub(/[0-9]/, '') if is_num
          check_password = check_password.gsub(Regexp.new(regxp_symbols), '') if is_sym
          r = false unless check_password.length == 0 #全ての文字が変換されない = 使用不可の文字が含まれている
        end
        return r
      end

      def validate_password_same_as_username(password, username)
        r = true
        is_permited_password_as_same_username = Sidfm.get_is_permited_password_as_same_username
        unless is_permited_password_as_same_username then
          if password == username then
            r = false
          end
        end  
        return r
      end
    
      def validate_password_includes_username(password, username)
        r = true
        is_permited_password_includes_username = Sidfm.get_is_permited_password_includes_username
        unless is_permited_password_includes_username then
          if password.include?(username) then
            r = false
          end
        end
        return r 
      end

      def check_password_with_password_policy
        if User.current.admin == true || !self.password then
          return true
        end
        #文字数のvalidation
        is_pass = validate_password_min_length self.password
        return nil unless is_pass
        #利用可能な記号のvalidation
        is_pass = validate_password_symbols self.password
        return nil unless is_pass  
        #利用可能な文字種のvalidation
        is_pass = validate_password_char_types self.password
        return nil unless is_pass
        if self.login then
          is_pass = validate_password_same_as_username self.password, self.login
          return nil unless is_pass
          is_pass = validate_password_includes_username self.password, self.login
          return nil unless is_pass
        end
        return true
      end
    end
  end
end

ActionDispatch::Reloader.to_prepare do
  unless User.included_modules.include?(SidfmPatches::UserPatch)
    User.send(:include, SidfmPatches::UserPatch)
  end
end
